*==============================================================================
* Program:				BREAKSTRINGSINTOLINES.PRG
* Purpose:				Speed test AT()/SUBSTR() vs. MLINE() vs. ALINES()
* From:				    We Used to Do it That Way, But
* Copyright:			(c) 2008 Tamar E. Granor, Ph.D.
* Last revision:		09/13/08
*==============================================================================

#DEFINE PASSES 1000

LOCAL cOriginal, nStart, nEnd, aContents1[1], aContents2[1], aContents3[1], nLine, nLines
LOCAL nLength, cLong, cShort

cLong = "Now is the time for all good men to come to the aid of their country."
cShort = "Just a note."

FOR nLength = 1 TO 10
	cOriginal = REPLICATE(m.cLong + CHR(13) + CHR(10) + m.cShort + CHR(13) + CHR(10), 100 * m.nLength)
	? "Original string length: ", LEN(cOriginal)

	* Oldest way is AT() and SUBSTR()
	nStart = SECONDS()
	FOR nPass = 1 TO PASSES
		cString = m.cOriginal
		* Find the first line break
		nBreak = AT(CHR(13) + CHR(10), m.cString)
		nLine = 0
		DO WHILE m.nBreak > 0
			nLine = m.nLine + 1 
			DIMENSION aContents3[m.nLine]
			aContents3[m.nLine] = LEFT(m.cString, m.nBreak-1)
			cString = SUBSTR(m.cString, m.nBreak + 2)
			nBreak = AT(CHR(13) + CHR(10), m.cString)
		ENDDO 
		IF NOT EMPTY(m.cString)
			nLine = m.nLine + 1 
			DIMENSION aContents3[m.nLine]
			aContents3[m.nLine] = m.cString
		ENDIF 
	ENDFOR 
	nEnd = SECONDS()
	? "  Using AT(), LEFT() and SUBSTR(), ", nEnd-nStart
	
	
	* With MLINE(), you have to do it one line at 
	* a time until you run out of lines.

	nStart = SECONDS()
	FOR nPass = 1 TO PASSES
		* Set memo width to max
		LOCAL nOldMemoWidth
		nOldMemoWidth = SET("Memowidth")
		SET MEMOWIDTH TO 1024

		nLines = MEMLINES(m.cOriginal)
		_MLINE=0
		DIMENSION aContents1[m.nLines]
		FOR nLine = 1 TO m.nLines
			aContents1[m.nLine] = MLINE(m.cOriginal, 1, _MLINE)
		ENDFOR
	ENDFOR 
	nEnd = SECONDS()
	? "  Using MLINE(), ", nEnd-nStart


	* With ALINES(), this is a one-liner
	nStart = SECONDS()
	FOR nPass = 1 TO PASSES
		nLines = ALINES(aContents2, m.cOriginal)
	ENDFOR 
	nEnd = SECONDS()
	? "  Using ALINES(), ", nEnd-nStart
ENDFOR 